/*
 * KernelConf - A Linux Kernel Configuration System
 *
 * Distributed under GNU General Public Lisence
 *
 * Copyright (C) 2002 by Anuradha Ratnaweera
 */

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include "tree.h"
#include "expressions.h"
#include "symbols.h"
#include "writeconfig.h"

extern MenuPtr root_menu_item, menu_tree_root;

#define MAX_MENU_NAME_LENGTH 256

#define MAX_INDENT 30

static char indent_string[MAX_INDENT+1];
static int rows, cols;

static int set_geometry()
{
    FILE *stty_out;
    system("stty size > stty-out 2> /dev/null");
    if ((stty_out = fopen("stty-out", "rt")) == NULL)
	return 1;
    fscanf(stty_out, "%d %d", &rows, &cols);
    fclose(stty_out);
    unlink("stty-out");
    return 0;
}

/* +1 increases, -1 decreases, 0 resets */
static void set_indent(int indent)
{
    static int indent_ptr = 0;

    indent_string[indent_ptr] = ' ';
    switch (indent) {
	case 1:
	    indent_ptr += 2;
	    break;
	case -1:
	    indent_ptr -= 2;
	    break;
	default:
	    indent_ptr = 0;
    }

    indent_string[indent_ptr] = 0;
}

static int show_help(MenuPtr menu_item)
{
    FILE *help_file, *mc_help;
    char *buffer;
    long bufsize;

    if ((bufsize = read_help(menu_item, &buffer)) < 0)
	return 1;

    if ((help_file = fopen("help.out", "wt")) == NULL)
	return 2;
    fwrite(buffer, sizeof(char), bufsize, help_file);
    free(buffer);
    fclose(help_file);

    if ((mc_help = fopen("MChelp", "wt")) == NULL)
	return 3;
    fprintf(mc_help, "./scripts/lxdialog/lxdialog --title '%s' --backtitle 'Linux Kernel Configuration' --textbox help.out %d %d",
	    menu_item->caption, rows - 1, cols - 1);
    fclose(mc_help);

    system("sh MChelp");

    unlink("help.out");
    unlink("MChelp");

    return 0;
}

static int do_menu(MenuPtr parent_menu)
{
    FILE *mc_menu, *mc_out;
    MenuPtr menu_item, current_item, option;
    char menu_name[MAX_MENU_NAME_LENGTH];
    int retval;
    char value;

    if (parent_menu->symbol)
	current_item = parent_menu;
    else
	current_item = parent_menu->child;
    retval = -1;

    do {
	set_indent(0);

	mc_menu = fopen("MCmenu", "wt");
	fprintf(mc_menu, "./scripts/lxdialog/lxdialog --title '%s' --backtitle 'Linux Kernel Configuration' --menu 'Arrow keys navigate the menu.  <Enter> selects submenus --->.  Highlighted letters are hotkeys.  Pressing <Y> includes, <N> excludes, <M> modularizes features.  Press <Esc><Esc> to exit, <?> for Help.  Legend: [*] built-in  [ ] excluded  <M> module  < > module capable' %d %d %d '%s'", parent_menu->caption, rows - 4, cols - 5, rows - 14, ((NodePtr)current_item)->name);

	if (parent_menu == root_menu_item)
	    menu_item = parent_menu->child;
	else {
	    menu_item = parent_menu;
	}

	while (menu_item) {
	    if ((menu_item != parent_menu) && menu_item->child) {
		if (menu_item->child->symbol &&
			(menu_item->child->symbol->type == TYPE_OPTION)) {
		    fprintf(mc_menu, " '%s' '(",
			    ((NodePtr)menu_item)->name);
		    for (option = menu_item->child; option;
			    option = (MenuPtr)(((NodePtr)option)->next)) {
			if (option->symbol->value.charval == 'y')
			    fprintf(mc_menu, "%s", option->caption);
		    }
		    fprintf(mc_menu, ") %s%s  --->'",
			    indent_string, menu_item->caption);
		}
		else {
		    fprintf(mc_menu, " '%s' '%s%s  --->'",
			    ((NodePtr)menu_item)->name,
			    indent_string, menu_item->caption);
		}
	    }
	    else {
		if (menu_item->symbol) {
		    switch (menu_item->symbol->type) {
			case TYPE_BOOL:
			    if (menu_item->symbol->value.charval == 'y')
				value = '*';
			    else
				value = ' ';
			    fprintf(mc_menu, " '%s' '[%c] %s%s'",
				    ((NodePtr)menu_item)->name, value,
				    indent_string, menu_item->caption);
			    if (menu_item == parent_menu)
				set_indent(1);
			    break;
			case TYPE_TRI:
			    if (menu_item->symbol->value.charval == 'y')
				value = '*';
			    else if (menu_item->symbol->value.charval == 'm')
				value = 'M';
			    else
				value = ' ';
			    fprintf(mc_menu, " '%s' '<%c> %s%s'",
				    ((NodePtr)menu_item)->name, value,
				    indent_string, menu_item->caption);
			    if (menu_item == parent_menu)
				set_indent(1);
			    break;
			case TYPE_INT:
			case TYPE_HEX:
			    break;
			case TYPE_CHAR:
			    break;
			case TYPE_STR:
			    break;
			case TYPE_OPTION:
			    for (option = menu_item; option;
				    option = (MenuPtr)(((NodePtr)option)->next)) {
				if (option->symbol->value.charval == 'y')
				    value = 'X';
				else
				    value = ' ';
				fprintf(mc_menu, " '%s' '(%c) %s%s'",
					((NodePtr)option)->name, value,
					indent_string, option->caption);
			    }
			    menu_item = NULL;
			    break;
			default:
			    break;
		    }
		}
		else {
		    fprintf(mc_menu, " '%s' '--- %s ---'", ((NodePtr)menu_item)->name, 
			    menu_item->caption);
		}
	    }

	    if (menu_item == parent_menu)
		menu_item = parent_menu->child;
	    else if (menu_item)
		menu_item = (MenuPtr)((NodePtr)menu_item)->next;

	    /* this is yet another quick and dirty hack ;) */
	    while (menu_item &&
		    !evaluate_expressions_yn(menu_item->depends,
			(NodePtr)(menu_item->symbol))) {
		if (menu_item->symbol && ((menu_item->symbol->type == TYPE_BOOL) || (menu_item->symbol->type == TYPE_TRI))) {
		    menu_item->symbol->value.charval = 'n';
		}
		menu_item = (MenuPtr)((NodePtr)menu_item)->next;
	    }
	}

	fprintf(mc_menu, "\n");
	fprintf(mc_menu, "exit $?\n");
	fclose(mc_menu);

	retval = system("sh MCmenu 2> MCout");
	unlink("MCmenu");

	mc_out = fopen("MCout", "rt");
	fscanf(mc_out, "%s", menu_name);
	fclose(mc_out);
	unlink("MCout");

	current_item = (MenuPtr)search_tree((NodePtr*)(&menu_tree_root),
		menu_name, NODE_SEARCH, sizeof(Menu), NULL);

	if ((retval == 0) || (retval == 768) || (retval == 1280)) {
	    if (current_item && current_item->child && (current_item != parent_menu))
		do_menu(current_item);
	}

	if (retval == 512)
	    show_help(current_item);

	if (current_item && current_item->symbol) {
	    switch (current_item->symbol->type) {
		case TYPE_BOOL:
		    if ((current_item == parent_menu) || !(current_item->child)) {
			if ((retval == 768) || (retval == 1280))
			    current_item->symbol->value.charval = 'y';
			else if (retval == 1024)
			    current_item->symbol->value.charval = 'n';
			else if (retval == 0) {
			    if (current_item->symbol->value.charval == 'y')
				current_item->symbol->value.charval = 'n';
			    else
				current_item->symbol->value.charval = 'y';
			}
		    }
		    break;
		case TYPE_TRI:
		    if ((current_item == parent_menu) || !(current_item->child)) {
			if (retval == 768)
			    current_item->symbol->value.charval = 'y';
			else if (retval == 1280)
			    current_item->symbol->value.charval = 'm';
			else if (retval == 1024)
			    current_item->symbol->value.charval = 'n';
			else if (retval == 0) {
			    if (current_item->symbol->value.charval == 'm')
				current_item->symbol->value.charval = 'y';
			    else if (current_item->symbol->value.charval == 'y')
				current_item->symbol->value.charval = 'n';
			    else
				current_item->symbol->value.charval = 'm';
			}
		    }
		    break;
		case TYPE_INT:
		case TYPE_HEX:
		    break;
		case TYPE_CHAR:
		    break;
		case TYPE_STR:
		    break;
		case TYPE_OPTION:
		    if ((retval == 0) || (retval == 768) || (retval == 1280)) {
			for (option = current_item->parent->child; option;
				option = (MenuPtr)(((NodePtr)option)->next)) {
			    if (option == current_item)
				option->symbol->value.charval = 'y';
			    else
				option->symbol->value.charval = 'n';
			}
			retval = 256;
		    }
		    break;
		default:
		    break;
	    }
	}

#ifdef DEBUG
	if (current_item == NULL)
	    printf("current_item NULL\n");
#endif

    } while ((retval != 256) && (retval != 65280));

    return retval;
}

int do_menuconfig()
{
    int i, retval;
    FILE *prompt_file;

    set_geometry();

    for (i = 0; i < MAX_INDENT; i++)
	indent_string[i] = ' ';

    do_menu(root_menu_item);

    prompt_file = fopen("MCprompt", "wt");
    fprintf(prompt_file, "./scripts/lxdialog/lxdialog --backtitle 'Linux Kernel Configuration' --yesno 'Do you wish to save your new kernel configuration' 5 %d\n", cols - 5);
    fprintf(prompt_file, "exit $?\n");
    fclose(prompt_file);

    retval = system("sh MCprompt");
    unlink("MCprompt");

    if (retval == 0) {
	printf("\n");
	printf("\n");
	printf("Saving your kernel configuration...");
	write_config(".config", "include/linux/autoconf.h");
	printf("done.\n");
	printf("\n");
	printf("*** End of Linux kernel configuration.\n");
	printf("*** Check the top-level Makefile for additional configuration.\n");
	printf("*** Next, you must run 'make dep'.\n");
	printf("\n");
    }
    else {
	printf("\n");
	printf("\n");
	printf("Your kernel configuration changes were NOT saved.\n");
	printf("\n");
    }

    return 0;
}

